(* AuxProj - auxiliary project library
 * Copyright (C) 1991,1993 Tomas Mandys-MandySoft
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *)

{ URL: http://www.2p.cz }

unit AuxProj;

{ AuxProj.htx }

interface
uses
  IniFiles, SysUtils
  {$IFNDEF LINUX}, Windows{$ENDIF}
  ;

function GetProgramPath: string;
procedure AdjustPath(var S: string; aSlash: Char = {$IFDEF LINUX}'/'{$ELSE}'\'{$ENDIF});

{$IFNDEF LINUX}
procedure WinExecAndWait(aCmd: string; aMode: Integer = SW_SHOW; aStdInput: THandle = INVALID_HANDLE_VALUE ; aStdOutput: THandle =INVALID_HANDLE_VALUE ; aStdErr: THandle =INVALID_HANDLE_VALUE ); { spusti program a ceka dokud neskonci cinnost }
function CheckShellExecute(Err: Integer): Integer;

function LocalMachine: string;
{$ENDIF}

function ScramblePass(const aPass: string): string;

procedure AssignIniFileName(aFileName: string; aMemIni: Boolean);

type
  EProjError = class(Exception)
  end;

procedure ProjError(const Msg: string);

var
  Ini: TCustomIniFile;
  UpdateIni: Boolean;

implementation
uses
  CmdLine{$IFDEF CLR}, System.Text{$ENDIF};

resourcestring
  sLastError = 'Error "%s": %.4x';

procedure ProjError(const Msg: string);
begin
  raise EProjError.Create(Msg);
end;

procedure AdjustPath;
begin
  if (S <> '') and not (S[Length(S)] in ['\',':','/']) then
    S:= S+aSlash;
end;

function GetProgramPath;
begin
  Result:= ExtractFilePath(ExpandFileName(ParamStr(0)));
  AdjustPath(Result);
end;

{$IFNDEF LINUX}
procedure WinExecAndWait;
var
  aStartupInfo: TStartupInfo;
  aProcessInformation: TProcessInformation;
  aExitCode: Cardinal;
  Attr: TSecurityAttributes;
  {$IFDEF CLR}
  SB: StringBuilder;
  {$ENDIF}
begin
  aCmd:= aCmd+#0;
  {$IFNDEF CLR}
  FillChar(aStartupInfo, SizeOf(aStartupInfo), 0);
  {$ENDIF}
  aStartupInfo.CB:= SizeOf(aStartupInfo);
  with aStartupInfo do
  begin
    dwFlags:= startf_UseShowWindow;
    wShowWindow:= aMode;
    hStdInput:= aStdInput;
    hStdOutput:= aStdOutput;
    hStdError:= aStdErr;
    if (hStdInput <> INVALID_HANDLE_VALUE ) or (hStdOutput <> INVALID_HANDLE_VALUE ) or (hStdError <> INVALID_HANDLE_VALUE ) then
      dwFlags:= dwFlags or STARTF_USESTDHANDLES;
  end;
  Attr.nLength:= SizeOf(Attr);
  Attr.lpSecurityDescriptor:= nil;
  Attr.bInheritHandle:= True;
  {$IFDEF CLR}
  SB:= StringBuilder.Create(Length(aCmd));
  SB.Append(aCmd);
  {$ENDIF}
  if not CreateProcess({$IFDEF CLR}''{$ELSE}nil{$ENDIF}, {$IFDEF CLR}SB{$ELSE}@aCmd[1]{$ENDIF}, {$IFNDEF CLR}@{$ENDIF}Attr, {$IFNDEF CLR}@{$ENDIF}Attr, True, Normal_Priority_Class, {$IFDEF CLR}''{$ELSE}nil{$ENDIF}, {$IFDEF CLR}''{$ELSE}nil{$ENDIF}, aStartupInfo, aProcessInformation) then
    RaiseLastWin32Error;
//    ProjError(Format(sLastError, ['CreateProcess', GetLastError]));
  repeat
    if not GetExitCodeProcess(aProcessInformation.hProcess, aExitCode) then
      RaiseLastWin32Error;
//      ProjError(Format(sLastError, ['GetExitCodeProcess', GetLastError]));
  until aExitCode <> Still_Active;
end;

function CheckShellExecute(Err: Integer): Integer;
resourcestring
  sShellExecuteErr = 'ShellExecute error: %d';
begin
  if Err <= 32 then
    ProjError(Format(sShellExecuteErr, [Err]));
  Result:= Err;
end;

function LocalMachine: string;
var
  Size: DWORD;
  {$IFDEF CLR}
  SB: StringBuilder;
  {$ELSE}
  S: array [0..MAX_COMPUTERNAME_LENGTH] of Char;
  {$ENDIF}
begin
  {$IFDEF CLR}
  Size:= MAX_COMPUTERNAME_LENGTH;
  SB:= StringBuilder.Create(Size);
  GetComputerName(SB, Size);
  Result:= SB.ToString;
  {$ELSE}
  Size := Sizeof(S);
  GetComputerName(S, Size);
  Result:= StrPas(S);
  {$ENDIF}
end;
{$ENDIF}

function ScramblePass(const aPass: string): string;
  procedure TestC(var C: string; I: Integer; C1, C2: Char);
  begin
    if (C[I] >= C1) and (C[I] <= C2) then
      C[I]:= Chr(Ord(C2) - (Ord(C[I])-Ord(C1)));
  end;
var
  I: Integer;
begin
  Result:= '';
  for I:= 1 to Length(aPass) do
    Result:= aPass[I]+Result;
  for I:= 1 to Length(Result) do
  begin
    TestC(Result, I, 'a', 'z');
    TestC(Result, I, 'A', 'Z');
    TestC(Result, I, '0', '9');
  end;
end;

procedure AssignIniFileName(aFileName: string; aMemIni: Boolean);
begin
  GetCmdString('I:', clUpcase or clValueCase, aFileName);
  aFileName:= ExpandFileName(aFileName);
  Ini.Free;
  if aMemIni then
    Ini:= TMemIniFile.Create(aFileName) // TMemIni file to support CD-ROM RO directories
  else
    Ini:= TIniFile.Create(aFileName);
  UpdateIni:= not IsLibrary;
end;

initialization
  AssignIniFileName(ChangeFileExt(ExpandFileName(ParamStr(0)), '.ini'), True);
finalization
  try
    if UpdateIni and not (Ini is TIniFile) then  // ToDo: update changes, changes made in a module (dll,ocx) rewrite each other
                                                 // in Kylix TIniFile is ancestor of TMemIniFile !!!
      Ini.UpdateFile;   // only if Modified, but Modified property does not exist
  except
  end;
  try
    Ini.Free;   // kylix do always UpdateFile
  except
  end;
  Ini:= nil;  // FreeAndNil(Ini); // not in D4
end.
